using UnityEngine;
using System.Collections;
using System.Collections.Generic;




public class Position : System.IComparable<Position> {
	
	public int x;
	
	
	public int y;
	
	public Position Rotated()
	{
		return new Position(y,-x);
	}
	public Position HFlipped()
	{
		return new Position(-x,y);
	}
	public Position VFlipped()
	{
		return new Position(x,-y);
	}
	
	public Position()
	{
	}
	
	public Position(int x, int y)
	{
		this.x=x;
		this.y=y;
	}
	
	public Position(Position p)
	{
		this.x=p.x;
		this.y=p.y;
	}
		
	public enum Direction {
		Up=0,
		Down=1,
		Left=2,
		Right=3,
		MAX=4,
	};
	
	public static Position[] DirPos = {
		new Position(0,-1),
		new Position(0,1),
		new Position(1,0),
		new Position(-1,0),
		new Position(0,0)
	};
	
	
	public void Translate(Position offset)
	{
		x+=offset.x;
		y+=offset.y;
	}
	
	public void Translate(Direction dir)
	{
		Translate(dir.ToPosition());
	}
	
	//EQUALITY BEGIN
	public override bool Equals(System.Object obj)
	{		
		return obj is Position && this == (Position)obj;
	}
	
	public override int GetHashCode()
	{
		int hash=17;
		hash = hash*29+x.GetHashCode();
		hash = hash*29+y.GetHashCode();
		return hash;
	}
	
	public static bool operator ==(Position x, Position y)
	{
		return x.x==y.x && x.y==y.y;
	}
	
	public static bool operator !=(Position x, Position y)
	{
		return x.x!=y.x || x.y!=y.y;
	}
	//EQUALITY END
	
	//ICOMPARABLE
	public int CompareTo(Position other)
	{
		if (x>other.x)
			return 1;
		else if (x<other.x)
			return -1;
		else if (y<other.y)
			return 1;
		else if (y>other.y)
			return -1;
		else
			return 0;
	}
	
	public static Position operator+ (Position a, Position b)
	{
		return new Position(a.x+b.x,a.y+b.y);
	}
	
	public static Position operator+ (Position a, Direction b)
	{
		return a+b.ToPosition();
	}
	
	
	public static Position operator- (Position a, Direction b)
	{
		return a+b.Invert().ToPosition();
	}
	
	public static Position operator* (int i, Position a)
	{
		return new Position(i*a.x,i*a.y);
	}
	
	public static Position operator+ (Direction a, Position b)
	{
		return a.ToPosition()+b;
	}
	
	// Override the ToString method to display an complex number in the suitable format:
   public override string ToString()
   {
      return(string.Format("({0},{1})", x, y));
   }
}
