using UnityEngine;
using System.Collections;
using System.Linq;
public class Editor : MonoBehaviour {
	
	public Texture2D[] imgs;
	
	private Level level;
	private RenderLevel rl;
	private Camera cam;
	private int selectedtool=0;
	
	private Rect[] rects;
	

	public bool IgnoreMouse()
	{		
    	Vector3 mousePos = Input.mousePosition;
    	mousePos.y = Screen.height - mousePos.y;
		return rects.Any(r=>r.Contains(mousePos));				
	}
	
	void OnGUI()
	{
		if (editoron)
		{
			selectedtool=GUI.SelectionGrid(
				new Rect (10, 40, 32, 32*imgs.Length),
				selectedtool,imgs,1);
		}
	}
	
	public static bool editoron;
	// Use this for initialization
	void Start () {
		editoron = Application.isEditor;
		rects = new Rect[]{
			new Rect (10, 40, 32, 32*imgs.Length),
			new Rect (10, 10, 200, 20)
		};
		
		level = GetComponent<Level>();
		cam = Camera.mainCamera;
		rl = GetComponent<RenderLevel>();
		oldzoom = Camera.main.orthographicSize;
	}

	Plane worldplane = new Plane(-Vector3.up,Vector3.zero);
	
	public Position cursorpos = new Position(0,0);
	public Vector3 cursorvect = Vector3.zero;
	
	bool dragging=false;
	
	bool resized=false;
	float oldzoom;
	
	Position dragfrom = new Position(0,0);
	// Update is called once per frame
	void Update () {	
		
		if (Input.GetKeyDown(KeyCode.E))
		{
			if (Level.alloweditor)
			{
				editoron=!editoron;
				Screen.showCursor=editoron;
			}
		}
		
		if (editoron==false)
		{
			return;
		}
		for (int i=1;i<=7;i++)
		{
			if (Input.GetKeyDown(i.ToString()))
			{
				selectedtool=i-1;
				dragging=false;
			}				
		}
		if (Input.GetKeyDown(KeyCode.B))
		{
			level.SetBounds();
		}
		
		if (Input.GetKeyDown(KeyCode.Space))
		{
			level.TakeUndoSnapshot();
			level.TryAutomate();
		}
		Ray r = cam.ScreenPointToRay(Input.mousePosition);
		float enter;
		worldplane.Raycast(r, out enter);
		Vector3 v = r.GetPoint(enter);
		cursorpos = rl.VectToPos(v);
		cursorvect = rl.PosToVect(cursorpos);
		
		switch(selectedtool)
		{
		case 0://
			RemoveToolTick();
			break;
		case 1://
			AddToolTick<Wall>();
			break;
		case 2://
			AddToolTick<Player>();
			break;
		case 3://
			AddBlockTick();
			break;
		case 4://
			AddToolTick<Whale>();
			break;
		case 5://
			AddToolTick<StartTile>();
			break;
		case 6://
			AddToolTick<EndTile>();
			break;
		}		
	}
	
	public float sensitivity;
	void FixedUpdate()
	{			
		if (editoron)
		{
			float mw = Input.GetAxis("Mouse ScrollWheel");
			cam.orthographicSize-=mw*Time.fixedDeltaTime*sensitivity;
		}
	}
	
	void AddBlockTick()
	{
		Position.Direction facing = 
			Input.GetKey(KeyCode.W) ? Position.Direction.Up :
			Input.GetKey(KeyCode.S) ? Position.Direction.Down :
			Input.GetKey(KeyCode.A) ? Position.Direction.Left :
			Input.GetKey(KeyCode.D) ? Position.Direction.Right :
				Position.Direction.MAX;
				
		bool removing = Input.GetKey(KeyCode.LeftShift)||Input.GetKey(KeyCode.RightShift);
			
		if (Input.GetMouseButtonDown(0)&&!IgnoreMouse())			
		{
			level.TakeUndoSnapshot();
		}
		
		if (Input.GetMouseButton(0)&&!IgnoreMouse())
		{
			if (removing)
			{
				level.RemoveEnts(cursorpos);
			}
			else
			{
				if (facing == Position.Direction.MAX)
				{
					Position p = new Position(cursorpos);
					Block b = new Block(p,facing,level.gamestate);
					level.AddEnt(b);
				}
				else
				{
					Position p1 = new Position(cursorpos);
					Position p2 = new Position(cursorpos+facing);
					Block b1 = new Block(p1,facing,level.gamestate);
					Block b2 = new Block(p2,facing.Invert(),level.gamestate);
					level.AddEnt(b1);
					level.AddEnt(b2);
				}
			}			
//			level.SetBounds();
		}
	}
	
	void AddToolTick<E>() where E : Entity
	{
		bool removing = Input.GetKey(KeyCode.LeftShift)||Input.GetKey(KeyCode.RightShift);
			
		if (Input.GetMouseButtonDown(0)&&!IgnoreMouse())			
		{
			level.TakeUndoSnapshot();
		}
		
		if (Input.GetMouseButton(0)&&!IgnoreMouse())
		{
			if (removing)
			{
				level.RemoveEnts(cursorpos);
			}
			else
			{
				Position p = new Position(cursorpos);
				if (System.Attribute.IsDefined(typeof(E),typeof(OnlyOneAttribute)))
				{
					for (int i=level.gamestate.entities.Count-1;i>=0;i--)
					{
						Entity e = level.gamestate.entities[i];
						if (e.GetType()==typeof(E))
						{							
							level.gamestate.RemoveEntity(e,false);				
						}
					}
				}
				
				if (typeof(E) == typeof(Whale) &&  (Input.GetKey(KeyCode.W)||Input.GetKey(KeyCode.S)||Input.GetKey(KeyCode.A)||Input.GetKey(KeyCode.D)))
				{					
					level.AddEnt(new StaticWhale(p,Position.Direction.MAX,level.gamestate));
				}
				else
				{
					Entity toadd = (Entity)System.Activator.CreateInstance(typeof(E),new object[]{p,Position.Direction.MAX,level.gamestate});				
					level.AddEnt(toadd);
				}
			}
//			level.SetBounds();
		}
	}
	
	void RemoveToolTick()
	{			
		if (Input.GetMouseButtonDown(0)&&!IgnoreMouse())			
		{
			level.TakeUndoSnapshot();
		}
		
		if (Input.GetMouseButton(0)&&!IgnoreMouse())
		{
			level.RemoveEnts(cursorpos);
			//level.SetBounds();
		}
	}
	
	void OnDrawGizmos()
	{		
		/*
		Gizmos.color=Input.GetKey(KeyCode.LeftShift) ? Color.red : Color.white;
		if (dragging)
		{
			Vector3 tl = cursorvect;
			Vector3 br = rl.PosToVect(dragfrom);
			Vector3 tr = new Vector3(tl.x,0,br.z);
			Vector3 bl = new Vector3(br.x,0,tl.z);
			
			Gizmos.DrawLine(tl,tr);
			Gizmos.DrawLine(tr,br);
			Gizmos.DrawLine(br,bl);
			Gizmos.DrawLine(bl,tl);
		}
		else
		{
			Gizmos.DrawWireSphere(cursorvect,Level.radius/2);
		}*/
	}
}
