using UnityEngine;
using System.Collections;
using System.IO;
using System.Collections.Generic;
using System.Linq;


public class LoaderSaver : MonoBehaviour {
	
	private Level level;
	[HideInInspector]
	public string levelname;
	
	public string[] levels;
	
     private static int ExtractNumber (string s)
     {
         string endnum_str = "";
         for (int i = s.Length - 1; i >= 0; i--)
         {
             System.Char c = s[i];
             if (System.Char.IsDigit(c))
             {
                 endnum_str = c.ToString() + endnum_str;
             }
             else
                 break;
         }
         if (endnum_str == "")
             return -1;
         else
             return int.Parse(endnum_str);
     }
	
     private void TryDecrementLevelName ()
     {
         levelname = levelname.Trim();
 
         int num = ExtractNumber(levelname);
         if (num == 0)
             return;
 
         if (num > 0)
         {
             string trimmed = levelname.TrimEnd('0', '1', '2', '3', '4', '5', '6', '7', '8', '9');
             levelname = trimmed + (num - 1).ToString();
         }
         else
         {
             levelname = levelname + "1";
         }
     }
 
     private void TryIncrementLevelName ()
     {
         levelname = levelname.Trim();
 
         int num = ExtractNumber(levelname);
 
         if (num >= 0)
         {
             string trimmed = levelname.TrimEnd('0', '1', '2', '3', '4', '5', '6', '7', '8', '9');
             levelname = trimmed + (num + 1).ToString();
         }
         else
         {
             levelname = levelname + "1";
         }
     }
	void OnGUI()
	{		
		if (Application.isEditor)
		{
    		levelname = GUI.TextField (new Rect (70, 10, 200, 20), levelname, 25);
//			PlayerPrefs.SetString("levelname",levelname);
		//}
		//else
		
		//{
			string ln = "Level " + levels.ToList().IndexOf(levelname);
			GUI.Label(new Rect(10,10,200,20), ln,style);
		}
	}
	
	private GUIStyle style;
	// Use this for initialization
	void Start () {
		
		style = new GUIStyle();
		style.normal.textColor=Color.black;
		levelname = PlayerPrefs.GetString("levelname",levels[0]);
		level = GetComponent<Level>();
		Load();
	}
	
	string FullPath()
	{
		return "Assets/Resources/Levels/"+levelname+".bytes";
	}
	
	string ResourcePath()
	{
		return "Levels/"+levelname;
	}
	
	// Update is called once per frame
	void Update () {
		if (Application.isEditor==false)
		{
			return;
		}
		
		if (Input.GetKeyDown(KeyCode.S))
		{
#if UNITY_EDITOR
			using (var filestream = System.IO.File.Open(FullPath(),FileMode.Create) )
			{
//				Debug.Log("saving " + levelname);
				level.gamestate.Write(filestream);									
				UnityEditor.AssetDatabase.Refresh(UnityEditor.ImportAssetOptions.ForceSynchronousImport | UnityEditor.ImportAssetOptions.ForceUpdate);
				PlayerPrefs.SetString("levelname",levelname);
       		}		
			SaveToBrowser();
#else
			SaveToBrowser();
#endif
			
		}
		if (Input.GetKeyDown(KeyCode.L))
		{
			if (Application.isEditor)
			{
				Load();
				PlayerPrefs.SetString("levelname",levelname);
			}
			else
			{
				Application.ExternalCall("LoadLevel");
			}
		}
		
		if (Input.GetKeyDown(KeyCode.Period))
		{
			TryIncrementLevelName();
			if (Application.isEditor==false)
				Load();
		}
		
		if (Input.GetKeyDown(KeyCode.Comma))
		{
			TryDecrementLevelName();
			if (Application.isEditor==false)
				Load();
		}
	}
	
	public void LoadNext()
	{
		int index = levels.ToList().IndexOf(levelname);
		if (index>=0)
		{
			index = index+1;
			levelname = levels[index];
			
			PlayerPrefs.SetString("levelname",levelname);
			if (levelname == levels[0])
				PlayerPrefs.DeleteKey("levelname");
			Load();
		}
	}
	
	public void SaveToBrowser()
	{		
		Debug.Log("saving to browser");
		using (var memstream = new System.IO.MemoryStream())
		{
			level.gamestate.Write(memstream);
			memstream.Position=0;
//				Debug.Log("saving " + levelname);
			byte[] b;
			using (var br = new System.IO.BinaryReader(memstream))
			{
				b = br.ReadBytes((int)memstream.Length);
			}		
			string dat_str = System.Convert.ToBase64String(b);
			Debug.Log("dat:\n"+dat_str);
			Application.ExternalCall("SaveLevel",dat_str);
   		}
	}
	
	public void LoadFromBrowser(string arg)
	{
		byte[] dat = System.Convert.FromBase64String(arg);
		using (Stream s = new MemoryStream(dat))
		{
			level.SetGameState( GameState.Read(s));		
			s.Position=0;
			level.SetBounds();
			GetComponent<RenderLevel>().SnapColors();				
		}
	}
	
	public void Load()
	{	
		if (Application.isEditor)
		{
			using (var filestream = System.IO.File.Open(FullPath(),FileMode.Open) )
			{
	//			Debug.Log("loading " + levelname);			
	//			Debug.Log("BL1 " + filestream.Length);
				level.SetGameState( GameState.Read(filestream));
				level.SetBounds();
				GetComponent<RenderLevel>().SnapColors();
			}
		}
		else
		{
			level.ClearUndos();
			TextAsset asset = (TextAsset)Resources.Load(ResourcePath());	
	//		Debug.Log(asset.text.Length);
	//		Debug.Log("BL2 " + asset.bytes.Length);
			using (Stream s = new MemoryStream(asset.bytes))
			{
				level.SetGameState( GameState.Read(s));			
				level.SetBounds();
				GetComponent<RenderLevel>().SnapColors();				
			}
		}
	}
		
}
