using UnityEngine;
using System.Collections;

public class MeshGen {
	
	
	public int meshindex=0;
	public Vector3[] vertices;
	public int[] tris;
	public Color[] colors;
	public Vector2[] uvs;
	
	public MeshGen()
	{
	}
	
	public void AssignTo(Mesh m)
	{		
		m.vertices=vertices;
		m.colors=colors;
		m.uv=uvs;
		m.triangles=tris;
		m.RecalculateBounds();
	}
	
	public void Alloc(int vertexcount, int tricount)
	{
		meshindex=0;
		
		if (vertices==null || vertices.Length!=vertexcount || tris.Length!=tricount)
		{		
			vertices = new Vector3[vertexcount];
			colors = new Color[vertexcount];
			uvs = new Vector2[vertexcount];
			tris = new int[tricount];
			
			for (int i=0;i<vertexcount/4;i++)
			{
				tris[6*i+0] = i*4+0;
				tris[6*i+1] = i*4+1;
				tris[6*i+2] = i*4+3;
				tris[6*i+3] = i*4+1;
				tris[6*i+4] = i*4+2;
				tris[6*i+5] = i*4+3;
			}				
			
		}
	}
	
	public void AddRing(Vector3 center, float r1, float r2, Color col)
	{
		Vector3 x = new Vector3(1,0,0);
		Vector3 z = new Vector3(0,0,1);
		Vector3 a2 = center-r1*x-r1*z;
		Vector3 b2 = center+r1*x-r1*z;
		Vector3 c2 = center+r1*x+r1*z;
		Vector3 d2 = center-r1*x+r1*z;
		Vector3 a1 = center-r2*x-r2*z;
		Vector3 b1 = center+r2*x-r2*z;
		Vector3 c1 = center+r2*x+r2*z;
		Vector3 d1 = center-r2*x+r2*z;
		
		
		AddQuad(a1,b1,b2,a2,col);		
		AddQuad(b1,c1,c2,b2,col);		
		AddQuad(c1,d1,d2,c2,col);		
		AddQuad(d1,a1,a2,d2,col);		
	}
	
	
	public void AddTile(Vector3 center, float radius, Color col)
	{
		Vector3 x = new Vector3(1,0,0);
		Vector3 z = new Vector3(0,0,1);
		Vector3 a = center-radius*x-radius*z;
		Vector3 b = center+radius*x-radius*z;
		Vector3 c = center+radius*x+radius*z;
		Vector3 d = center-radius*x+radius*z;
		AddQuad(a,b,c,d,col);		
	}
	
	
	public void AddFacingTile(Vector3 center, float r1, float r2, Position.Direction facing, Color col)
	{
		Vector3 a,b,c,d;
		Vector3 x = new Vector3(1,0,0);
		Vector3 z = new Vector3(0,0,1);
		switch (facing)
		{
		case Position.Direction.Up:
		{
			a = center-r2*x-r1*z;//ul
			b = center+r1*x-r1*z;//dl
			c = center+r1*x+r1*z;//dr
			d = center-r2*x+r1*z;//ur
			break;
		}
		case Position.Direction.Down:
		{
			a = center-r1*x-r1*z;//ul
			b = center+r2*x-r1*z;//dl
			c = center+r2*x+r1*z;//dr
			d = center-r1*x+r1*z;//ur
			break;
		}
		case Position.Direction.Left:
		{
			a = center-r1*x-r1*z;//ul
			b = center+r1*x-r1*z;//dl
			c = center+r1*x+r2*z;//dr
			d = center-r1*x+r2*z;//ur
			break;
		}
		case Position.Direction.Right:
		default://just because
		{
			a = center-r1*x-r2*z;//ul
			b = center+r1*x-r2*z;//dl
			c = center+r1*x+r1*z;//dr
			d = center-r1*x+r1*z;//ur
			break;
		}
		}
		AddQuad(a,b,c,d,col);		
	}
	
	public void AddQuad(Vector3 a, Vector3 b, Vector3 c, Vector3 d, Color col)
	{
		try
		{
			vertices[meshindex*4+0]=a;
			vertices[meshindex*4+1]=b;
			vertices[meshindex*4+2]=c;
			vertices[meshindex*4+3]=d;
			
			colors[meshindex*4+0]=col;
			colors[meshindex*4+1]=col;
			colors[meshindex*4+2]=col;
			colors[meshindex*4+3]=col;
			
			meshindex++;
		}
		catch
		{
		}
	}
	
}
